/**
 * AREST - RESTful Authentication for PHP 4/5
 * Version: 1.2.1
 * Author: Travis Estill (travisce.com)
 * Copyright (c) 2010 Travis Estill
 * License: Distributed under the Lesser General Public License
 * (LGPL) http://www.gnu.org/copyleft/lesser.html
 * This program is distributed in the hope that it will be useful
 * - WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

var arest = {
  accept : 'text/plain, text/html;q=0.9, '
    + 'application/xhtml+xml;q=0.9, application/xml;q=0.9, '
    + '*/*;q=0.8',
  
  addSlashes : function(str) {
    str = str.replace(/\\/g, '\\\\');
    str = str.replace(/\'/g, '\\\'');
    str = str.replace(/\"/g, '\\"');
    str = str.replace(/\0/g, '\\0');
    return str;
  },
  
  authRequest : function(user, realm, nonce, cnonce, rsp, uri) {
    var auth, 
        encAuth, 
        params, 
        xhr;
    
    // Configure Digest directives
    auth = 'username="' + arest.addSlashes(user) + '", ';
    auth += 'realm="' + arest.addSlashes(realm) + '", ';
    auth += 'nonce="' + arest.addSlashes(nonce) + '", ';
    auth += 'cnonce="' + arest.addSlashes(cnonce) + '", ';
    auth += 'opaque="' + arest.addSlashes(arest.opaque) + '", ';
    auth += 'response="' + arest.addSlashes(rsp) + '", ';
    auth += 'qop="auth-basic"';
    encAuth = Base64.encode(auth);
    
    // Authentication request
    params = 'arest-auth-scheme=Basic';
    params += '&arest-response=1';
    xhr = new XMLHttpRequest();
    xhr.open('POST', uri, false, arest.randStr(), encAuth);
    xhr.setRequestHeader('Accept', arest.accept);
    xhr.setRequestHeader('Content-Type', 
      'application/x-www-form-urlencoded');
    xhr.setRequestHeader('Content-Length', params.length);
    xhr.setRequestHeader('Connection', 'close');
    xhr.send(params);
    
    return xhr;
  },
  
  configAuthForm : function() {
    var authChallenge,
        authDL,
        authFieldset,
        authForm,
        passDD,
        passDT,
        passInput,
        passLabel,
        realms,
        userDD,
        userDT,
        userInput,
        userLabel;
    
    if (typeof window.opera == 'object'
          || typeof window.devicePixelRatio != 'undefined') {
      return false;
    }
    
    if (typeof XMLHttpRequest == 'undefined') {
      return false;
    }
    
    if (typeof arest.accept == 'undefined'
          || typeof arest.domain == 'undefined'
          || typeof arest.realm == 'undefined'
          || typeof arest.opaque == 'undefined'
          || typeof arest.nonce == 'undefined'
          || typeof arest.msg == 'undefined') {
      return false;
    }
    
    authForm = document.getElementById('arest-auth');
    authFieldset = authForm.getElementsByTagName('fieldset').item(0);
    authChallenge = document.getElementById('arest-challenge');
    
    // Create username/password inputs
    userDT = document.createElement('dt');
    userLabel = document.createElement('label');
    userLabel.htmlFor = 'arest-user';
    userLabel.appendChild(document.createTextNode('Username'));
    userDT.appendChild(userLabel);
    
    userDD = document.createElement('dd');
    userInput = document.createElement('input');
    userInput.id = 'arest-user';
    userInput.name = 'arest-user';
    userInput.type = 'text';
    userDD.appendChild(userInput);
    
    passDT = document.createElement('dt');
    passLabel = document.createElement('label');
    passLabel.htmlFor = 'arest-pass';
    passLabel.appendChild(document.createTextNode('Password'));
    passDT.appendChild(passLabel);
    
    passDD = document.createElement('dd');
    passInput = document.createElement('input');
    passInput.id = 'arest-pass';
    passInput.name = 'arest-pass';
    passInput.type = 'password';
    passDD.appendChild(passInput);
    
    authDL = document.createElement('dl');
    authDL.appendChild(userDT);
    authDL.appendChild(userDD);
    authDL.appendChild(passDT);
    authDL.appendChild(passDD);
    authFieldset.insertBefore(authDL, authChallenge);
    
    authForm.onsubmit = arest.processAuthForm;
  },
  
  crossDomainAuth : function(user, ha1) {
    var absURI,
        cnonce,
        domain,
        fullURI,
        i,
        iframe,
        iframeContainer,
        params,
        reqURI,
        response,
        timeVal,
        uri;
    
    timeVal = String(Math.round(new Date().getTime() / 1000));
    cnonce = arest.randStr();
    response = MD5(ha1 + ':' + arest.nonce + ':' + cnonce);
    
    params = 'arest-time=' + encodeURIComponent(timeVal);
    params += '&arest-user=' + encodeURIComponent(user);
    params += '&arest-realm=' + encodeURIComponent(arest.realm);
    params += '&arest-nonce=' + encodeURIComponent(arest.nonce);
    params += '&arest-cnonce=' + encodeURIComponent(cnonce);
    params += '&arest-rsp=' + encodeURIComponent(response);
    
    iframeContainer = document.createElement('div');
    iframeContainer.id = 'arest-domain';
    document.body.appendChild(iframeContainer);
    
    fullURI = window.location.protocol + '//'
      + window.location.hostname + ':'
      + (window.location.port == '' ? 80 : window.location.port)
      + window.location.pathname
      + window.location.search
      + window.location.hash;
    absURI = window.location.href;
    reqURI = window.location.pathname + window.location.search;
    
    domain = String(arest.domain).split(' ');
    for (i = 0; i < domain.length; i++) {
      // Skip URI if it matches the current location
      if (domain[i] == fullURI || domain[i] == absURI
            || domain[i] == reqURI) {
        continue;
      }
      
      // Add parameters to query string
      if (domain[i].indexOf('?') == -1) {
        if (domain[i].indexOf('#') != -1) {
          uri = domain[i].replace('#', '?' + params + '#');
        } else {
          uri = domain[i] + '?' + params;
        }
      } else {
        uri = domain[i].replace('?', '?' + params + '&');
      }
      
      iframe = document.createElement('iframe');
      iframe.src = uri;
      iframe.width = 0;
      iframe.height = 0;
      iframe.style.border = 0;
      iframeContainer.appendChild(iframe);
      
      if (iframe.attachEvent) {
        iframe.attachEvent('onload', function() {
          iframe.className = 'loaded';
        });
      } else {
        iframe.onload = function() {
          iframe.className = 'loaded';
        }
      }
    }
    
    // Allow 10 seconds for IFRAMEs to load
    window.setTimeout(function() {
      window.location.href = window.location.href;
    }, 10000);
    
    window.setInterval(function() {
      var child,
          domain,
          i;
      
      domain = document.getElementById('arest-domain');
      for (i = 0; i < domain.childNodes.length; i++) {
        child = domain.childNodes.item(i);
        if (child.nodeType == 1 && child.tagName == 'IFRAME') {
          if (child.className != 'loaded') {
            return false;
          }
        }
      }
      
      // IFRAMEs have been loaded; refresh page
      window.location.href = window.location.href;
    }, 1000);
    
    return true;
  },
  
  processAuthForm : function() {
    try {
      var cnonce,
          ha1,
          header,
          msg,
          msgText,
          p,
          pass,
          params,
          response,
          statusClass,
          uri,
          user,
          xhr;
      
      // Show loading message
      if (document.getElementById('arest-message')) {
        msg = document.getElementById('arest-message');
        msg.innerHTML = arest.msg.loading;
      }
      
      user = String(document.getElementById('arest-user').value);
      pass = String(document.getElementById('arest-pass').value);
      
      // Avoid sending credentials in plaintext
      document.getElementById('arest-user').value = '';
      document.getElementById('arest-pass').value = '';
      
      // Authentication request
      ha1 = MD5(user + ':' + arest.realm + ':' + pass);
      cnonce = arest.randStr();
      response = MD5(ha1 + ':' + arest.nonce + ':' + cnonce);
      uri = arest.updateURITime(this.action);
      xhr = arest.authRequest(user, arest.realm, arest.nonce, 
        cnonce, response, uri);
      
      // Repeat request if nonce has changed (delay in user activity)
      header = xhr.getResponseHeader('X-Retry-Nonce');
      if (!!header && header != arest.nonce) {
        arest.nonce = header; // Update nonce
        cnonce = arest.randStr();
        response = MD5(ha1 + ':' + arest.nonce + ':' + cnonce);
        uri = arest.updateURITime(this.action);
        xhr = arest.authRequest(user, arest.realm, arest.nonce, 
          cnonce, response, uri);
        
        // Updated nonce should match the nonce from the last request
        header = xhr.getResponseHeader('X-Retry-Nonce');
        if (!!header && header != arest.nonce) {
          throw "Invalid Nonce";
        }
      }
      
      // Status code handling
      statusClass = Number(String(xhr.status).charAt(0));
      if (statusClass == 2) {
        // Send 2nd request to ensure browser remembers credentials
        params = 'arest-response=1';
        xhr = new XMLHttpRequest();
        xhr.open('POST', arest.updateURITime(this.action), false);
        xhr.setRequestHeader('Accept', arest.accept);
        xhr.setRequestHeader('Content-Type', 
          'application/x-www-form-urlencoded');
        xhr.setRequestHeader('Content-Length', params.length);
        xhr.setRequestHeader('Connection', 'close');
        xhr.send(params);
        
        statusClass = Number(String(xhr.status).charAt(0));
        if (statusClass == 2) {
          // Login successful
          if (!arest.crossDomainAuth(user, ha1)) {
            window.location.href = window.location.href;
          }
        } else {
          throw "Unexpected response status";
        }
      } else if (statusClass == 3) {
        header = xhr.getResponseHeader('Location');
        if (!!header) {
          window.location.href = header;
        } else {
          window.location.href = this.action;
        }
      } else if (xhr.status == 401 || xhr.status == 403) {
        header = xhr.getResponseHeader('Content-Type');
        if (header.substr(0, 10) == 'text/plain') {
          msgText = xhr.responseText;
          // Remove status line
          msgText = msgText.substr(msgText.indexOf('\n'));
        } else {
          msgText = xhr.status + ' ' + xhr.statusText;
        }
        
        if (document.getElementById('arest-message')) {
          msg = document.getElementById('arest-message');
          while (msg.hasChildNodes()) {
            msg.removeChild(msg.firstChild);
          }
          p = msg.appendChild(document.createElement('p'));
          p.className = 'error';
          p.appendChild(document.createTextNode(msgText));
        } else {
          alert(msgText);
        }
      } else {
        throw "Unexpected response status";
      }
      
      return false;
    } catch (e) {
      // Clear credentials and send form normally
      document.getElementById('arest-user').value = '';
      document.getElementById('arest-pass').value = '';
      return true;
    }
  },
  
  queryAuth : function() {
    var params,
        pattern,
        query,
        queryRegExp,
        result,
        uri;
    
    query = String(window.location.search);
    pattern = 'arest-(user|realm|cnonce|rsp)=([^&=]+)';
    queryRegExp = new RegExp(pattern, 'g');
    params = new Object();
    do {
      result = queryRegExp.exec(query);
      if (result != null) {
        if (typeof result[1] == 'string'
              && typeof result[2] == 'string') {
          params[result[1]] = decodeURIComponent(result[2]);
        }
      }
    } while (result);
    
    if (typeof params['user'] == 'undefined'
          || typeof params['realm'] == 'undefined'
          || typeof params['cnonce'] == 'undefined'
          || typeof params['rsp'] == 'undefined') {
      return false;
    }
    
    // Authentication request
    uri = String(window.location.href);
    uri = uri.replace(
      /arest-(user|realm|nonce|cnonce|rsp)=[^&=]*&?/g, '');
    uri = arest.updateURITime(uri);
    arest.authRequest(params['user'], params['realm'], arest.nonce, 
      params['cnonce'], params['rsp'], uri);
    
    return true;
  },
  
  randStr : function() {
    var chars,
        i,
        randNum,
        randStr,
        strLen;
    
    chars = '0123456789-._';
    chars += 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    chars += 'abcdefghijklmnopqrstuvwxyz';
    
    strLen = 16;
    randStr = '';
    for (i = 0; i < strLen; i++) {
      randNum = Math.floor(Math.random() * chars.length);
      randStr += chars.substring(randNum, randNum + 1);
    }
    
    return randStr;
  },
  
  updateURITime : function(uri) {
    var timeParam,
        updated;
    
    timeParam = 'arest-time=';
    timeParam += String(Math.round(new Date().getTime() / 1000));
    updated = uri.replace(/arest-time=[0-9]+/g, timeParam);
    return updated;
  }
}
