<?php
/**
 * AREST - RESTful Authentication for PHP 4/5
 * Version: 1.2.1
 * Author: Travis Estill (travisce.com)
 * Copyright (c) 2010 Travis Estill
 * License: Distributed under the Lesser General Public License
 * (LGPL) http://www.gnu.org/copyleft/lesser.html
 * This program is distributed in the hope that it will be useful
 * - WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

///////////////////////////////////////
// DATABASE FUNCTIONS
///////////////////////////////////////

/**
 * Adds an authentication nonce to the database
 * @param   string    $nonce    encrypted nonce
 * @param   int|null  $user_id  associated user ID or NULL
 * @param   int       $expire   expiration time (UNIX timestamp)
 * @return  bool                TRUE on success or FALSE on failure
 */
function arest_db_add_nonce($nonce, $user_id, $expire) {
  global $arest_mysql;
  
  $sql_nonce = mysql_real_escape_string($nonce);
  $sql_user_id = is_null($user_id) ? 'NULL' : (int) $user_id;
  $sql_expire = (int) $expire;
  
  $query = 'INSERT INTO arest_nonces (nonce, user_id, expiration) ';
  $query .= 'VALUES("' . $sql_nonce . '", ' . $sql_user_id . ', ';
  $query .= 'FROM_UNIXTIME(' . $sql_expire . '))';
  return mysql_query($query, $arest_mysql);
}

/**
 * Deletes old authentication nonces
 * @return  null
 */
function arest_db_delete_old_nonces() {
  global $arest_mysql;
  
  $query = 'DELETE FROM arest_nonces ';
  $query .= 'WHERE expiration ';
  $query .= '< DATE_SUB(NOW(), INTERVAL ';
  $query .= (int) AREST_NONCE_LIFETIME . ' DAY) ';
  $query .= 'LIMIT 5000';
  mysql_query($query, $arest_mysql);
}

/**
 * Disables an authentication nonce in the database
 * @param   string  $nonce  encrypted auth nonce
 * @return  bool            TRUE on success or FALSE on failure
 */
function arest_db_disable_nonce($nonce) {
  global $arest_mysql;
  
  $query = 'UPDATE arest_nonces ';
  $query .= 'SET disabled = TRUE ';
  $query .= 'WHERE nonce = ';
  $query .= '"' . mysql_real_escape_string($nonce) . '"';
  return mysql_query($query, $arest_mysql);
}

/**
 * Retrieves a user's Digest HA1
 * @param   int          $user   user ID
 * @param   string       $realm  realm
 * @return  string|bool          hash on success or FALSE on failure
 */
function arest_db_get_digest_ha1($user_id, $realm) {
  global $arest_mysql;
  
  $sql_user_id = (int) $user_id;
  $sql_realm = mysql_real_escape_string($realm);
  
  $query = 'SELECT digest_ha1 ';
  $query .= 'FROM arest_digests ';
  $query .= 'WHERE realm = "' . $sql_realm . '"';
  $query .= '&& user_id = ' . $sql_user_id;
  $result = mysql_query($query, $arest_mysql);
  if (is_resource($result) && mysql_num_rows($result) == 1) {
    return mysql_result($result, 0);
  } else {
    return FALSE;
  }
}

/**
 * Retrieves the data from an authentication nonce
 * @param   string      $nonce  encrypted nonce
 * @return  array|bool          nonce data (user_id, expiration, 
 *                              disabled, nc) on success 
 *                              or FALSE on failure
 */
function arest_db_get_nonce($nonce) {
  global $arest_mysql;
  
  $sql_nonce = mysql_real_escape_string($nonce);
  
  $query = 'SELECT user_id, ';
  $query .= 'UNIX_TIMESTAMP(expiration) AS expiration, ';
  $query .= 'disabled, ';
  $query .= 'nc ';
  $query .= 'FROM arest_nonces ';
  $query .= 'WHERE nonce = "' . $sql_nonce . '"';
  $result = mysql_query($query, $arest_mysql);
  if (is_resource($result) && mysql_num_rows($result) == 1) {
    return mysql_fetch_assoc($result);
  } else {
    return FALSE;
  }
}

/**
 * Retrieves the ID of a username
 * @param   string    $user  username
 * @return  int|bool         user ID on success or FALSE on failure
 */
function arest_db_get_user_id($user) {
  global $arest_mysql;
  
  $query = 'SELECT user_id ';
  $query .= 'FROM arest_users ';
  $query .= 'WHERE username = "';
  $query .= mysql_real_escape_string($user) . '"';
  $result = mysql_query($query, $arest_mysql);
  if (is_resource($result) && mysql_num_rows($result) == 1) {
    return mysql_result($result, 0);
  } else {
    return FALSE;
  }
}

/**
 * Updates an authentication nonce
 * @param   int|null  $user_id  user ID or NULL
 * @param   int       $expire   expiration time (UNIX timestamp)
 * @param   int       $nc       nounce count
 * @param   string    $nonce    encrypted nonce
 * @return  bool                TRUE on success or FALSE on failure
 */
function arest_db_update_nonce($user_id, $expire, $nc, $nonce) {
  global $arest_mysql;
  
  $sql_user_id = is_null($user_id) ? 'NULL' : (int) $user_id;
  $sql_expire = (int) $expire;
  $sql_nc = (int) $nc;
  $sql_nonce = mysql_real_escape_string($nonce);
  
  $query = "UPDATE arest_nonces ";
  $query .= "SET user_id = {$sql_user_id}, ";
  $query .= "expiration = FROM_UNIXTIME({$sql_expire}), ";
  $query .= "nc = {$sql_nc} ";
  $query .= "WHERE nonce = '{$sql_nonce}'";
  return mysql_query($query, $arest_mysql);
}
?>
